'use client';

import React, { useState, useEffect, useRef } from 'react';
import breakpoints from './breakpoints';

export interface ResponsiveVideoProps
  extends React.VideoHTMLAttributes<HTMLVideoElement> {
  src: string;
  /** first match in the array will be selected
   * media should be a valid media query or
   * a key from breakpoints object
   */
  srcSet?: {
    src: string;
    media: string;
  }[];
  /** auto plays video source */
  autoPlay?: boolean;
  /** If autoplay fails and if there is a fallback,
   * display this fallback element */
  autoPlayFallback?: React.ReactNode;
}

const ResponsiveVideo = ({
  src,
  srcSet,
  autoPlay,
  autoPlayFallback,
  ...rest
}: ResponsiveVideoProps) => {
  const [isAutoPlayFailed, setIsAutoPlayFailed] = useState(false);
  const [currentSrc, setCurrentSrc] = useState(src);
  const videoRef = useRef<HTMLVideoElement>(null);

  useEffect(() => {
    async function play() {
      try {
        autoPlay && (await videoRef?.current?.play());
      } catch (err) {
        setIsAutoPlayFailed(true);
      }
    }
    play();
  }, [autoPlay, currentSrc]);

  useEffect(() => {
    const handleResize = () => {
      const matchedSet = srcSet?.find(
        ({ media }) => window.matchMedia(breakpoints[media] || media).matches
      );
      setCurrentSrc(matchedSet?.src || src);
    };
    window.addEventListener('resize', handleResize);
    handleResize();
    return () => window.removeEventListener('resize', handleResize);
  }, [src, srcSet]);

  return isAutoPlayFailed && autoPlayFallback ? (
    autoPlayFallback
  ) : (
    <video ref={videoRef} src={currentSrc} {...rest} />
  );
};

export default ResponsiveVideo;
